package uk.plc.graphic.classes;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Calendar;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Vector;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import uk.plc.graphic.database.DatabaseConnection;

public class Page implements FolderItem{

    private String uri;
    private String title;
    private String name;
    private int columns;
    private int identifier;
    private Page parentPage;
    private boolean checkedParentPage = false;
    private String parentPageUri;
    private static Pattern REGEX_FILENAME;
    private HashMap<String, TextBlock> textblocks;

    private HashMap<Integer, Vector<TextBlock>> textblocksInColumns;

    public String getUri() {
        return uri;
    }

    public String getTitle() {
        return title;
    }

    public int getColumns() {
        return columns;
    }

    public Page getParentPage() throws SQLException {
        if (checkedParentPage) {
            return parentPage;
        }
        this.parentPage = Page.getPage(parentPageUri);
        checkedParentPage = true;
        return parentPage;
    }

    public void setTitle(String title) {
        this.title = title;
    }

    public void setColumns(int columns) {
        this.columns = columns;
    }

    public void setParentPage(Page parentPage) {
        this.parentPageUri = parentPage.getUri();
        this.parentPage = parentPage;
    }

    public Page(String uri, String title, String name, int columns, String parentPageUri, int identifier) {
        this.uri = uri;
        this.title = title;
        this.name = name;
        this.columns = columns;
        this.parentPageUri = parentPageUri;
        this.identifier = identifier;
    }

    public TextBlock getBlock(String key) throws SQLException {
        if (textblocks == null) {
            getTextBlocks();
        }
        return textblocks.get(key);
    }
    public TextBlock getBlock(int col, int pos) throws SQLException {
        if (textblocksInColumns == null) {
            getTextBlocks();
        }
        Vector<TextBlock> blocks =  textblocksInColumns.get(col);
        TextBlock found = null;
        TextBlock block;
        Iterator<TextBlock> iter = blocks.iterator();
        while(iter.hasNext() && found == null){
            block = iter.next();
            if(block.getPosition()==pos) found = block;
        }
        return found;
    }
    public Vector<TextBlock> getTextBlocksColumn(int col) throws SQLException {
        if (this.textblocksInColumns != null) {
            return this.textblocksInColumns.get(col);
        }
        getTextBlocks();
        return this.textblocksInColumns.get(col);
    }
    public HashMap<Integer, Vector<TextBlock>> getTextBlocksInColumns() throws SQLException {
        if (this.textblocksInColumns != null) {
            return this.textblocksInColumns;
        }
        getTextBlocks();
        return this.textblocksInColumns;
    }
    public HashMap<String, TextBlock> getTextBlocks() throws SQLException {
        if (textblocks != null) {
            return textblocks;
        }
        this.textblocks = new HashMap<String, TextBlock>();
        this.textblocksInColumns = new HashMap<Integer, Vector<TextBlock>>();
        Connection conn = null;
        PreparedStatement preparedStatement = null;
        ResultSet result = null;
        try {
            conn = DatabaseConnection.getConnection();
            preparedStatement = conn.prepareStatement("SELECT * FROM `textblocks` WHERE page_ref = ? ORDER BY `column`,`position` ASC");
            preparedStatement.setString(1, uri);
            result = preparedStatement.executeQuery();
            TextBlock block;
            while (result.next()) {
                String text = result.getString("data");
                String key = result.getString("key");
                String block_title = result.getString("title");
                int position = result.getInt("position");
                int column = result.getInt("column");
                int id = result.getInt("id");
                block = new TextBlock(id, text, key, block_title, column, position);
                this.textblocks.put(key, block);
                if(this.textblocksInColumns.get(column)==null){
                    this.textblocksInColumns.put(column, new Vector<TextBlock>());
                }
                this.textblocksInColumns.get(column).add(block);
            }
            for(Integer col : this.textblocksInColumns.keySet()){
                Collections.sort(this.textblocksInColumns.get(col), new Comparator<TextBlock>(){
                    public int compare(TextBlock o1, TextBlock o2) {
                        return o1.getPosition()-o2.getPosition();
                    }
                });
            }
            return this.textblocks;
        } finally {
            DatabaseConnection.closePreparedStatement(preparedStatement);
            DatabaseConnection.closeResultSet(result);
            DatabaseConnection.closeConnection(conn);
        }
    }

    public static Page getPageByIdentifier(int identifier) throws SQLException {
        Connection conn = null;
        PreparedStatement preparedStatement = null;
        ResultSet result = null;
        try {
            conn = DatabaseConnection.getConnection();
            preparedStatement = conn.prepareStatement("SELECT * FROM `pages` WHERE identifier = ?");
            preparedStatement.setInt(1, identifier);
            result = preparedStatement.executeQuery();
            if (result.next()) {
                String uri = result.getString("uri");
                String title = result.getString("title");
                String name = result.getString("name");
                int columns = result.getInt("columns");
                String parentPage = result.getString("parent_page");
                return new Page(uri, title, name, columns, parentPage, identifier);
            }
            return null;
        } finally {
            DatabaseConnection.closePreparedStatement(preparedStatement);
            DatabaseConnection.closeResultSet(result);
            DatabaseConnection.closeConnection(conn);
        }
    }

    public static Page getPage(String uri) throws SQLException {
        Connection conn = null;
        PreparedStatement preparedStatement = null;
        ResultSet result = null;
        try {
            conn = DatabaseConnection.getConnection();
            preparedStatement = conn.prepareStatement("SELECT * FROM `pages` WHERE uri = ?");
            preparedStatement.setString(1, uri);
            result = preparedStatement.executeQuery();
            if (result.next()) {
                String title = result.getString("title");
                int columns = result.getInt("columns");
                int identifier = result.getInt("identifier");
                String parentPage = result.getString("parent_page");
                String name = result.getString("name");
                return new Page(uri, title, name, columns, parentPage, identifier);
            }
            return null;
        } finally {
            DatabaseConnection.closePreparedStatement(preparedStatement);
            DatabaseConnection.closeResultSet(result);
            DatabaseConnection.closeConnection(conn);
        }
    }

    public static Page insertPage(String uri, String title, String name, int columns, 
            String parentPage) throws SQLException {
        Connection conn = null;
        PreparedStatement preparedStatement = null;
        ResultSet result = null;
        try {
            conn = DatabaseConnection.getConnection();
            preparedStatement = conn.prepareStatement("INSERT INTO `pages` "
                    + "(`uri`,`title`,`name`, `parent_page`, `columns`) VALUES(?,?, ?, ?, ?");
            preparedStatement.setString(1, uri);
            preparedStatement.setString(2, title);
            preparedStatement.setString(3, name);
            preparedStatement.setString(4, parentPage);
            preparedStatement.setInt(5, columns);
            preparedStatement.execute();
            preparedStatement = conn.prepareStatement("SELECT `identifier` FROM `pages` WHERE uri = ?");
            preparedStatement.setString(1, uri);
            result = preparedStatement.executeQuery();
            int identifier = -1;
            if (result.next()) {
                identifier = result.getInt("identifier");
            }
            return new Page(uri, title, name, columns, parentPage, identifier);
        } finally {
            DatabaseConnection.closePreparedStatement(preparedStatement);
            DatabaseConnection.closeResultSet(result);
            DatabaseConnection.closeConnection(conn);
        }
    }

    public static Page[] getPages() throws SQLException {
        Connection conn = null;
        PreparedStatement preparedStatement = null;
        ResultSet result = null;
        try {
            conn = DatabaseConnection.getConnection();
            Vector<Page> pages = new Vector<Page>();
            preparedStatement = conn.prepareStatement("SELECT * FROM `pages`");
            result = preparedStatement.executeQuery();
            String page_uri;
            String page_title;
            String page_name;
            int page_columns;
            int page_identifier;
            String page_parentPage;
            while (result.next()) {
                page_uri = result.getString("uri");
                page_title = result.getString("title");
                page_name = result.getString("name");
                page_columns = result.getInt("columns");
                page_parentPage = result.getString("parent_page");
                page_identifier = result.getInt("identifier");
                pages.add(new Page(page_uri, page_title, page_name, page_columns, page_parentPage, page_identifier));
            }
            return pages.toArray(new Page[pages.size()]);
        } finally {
            DatabaseConnection.closePreparedStatement(preparedStatement);
            DatabaseConnection.closeResultSet(result);
            DatabaseConnection.closeConnection(conn);
        }
    }
    public Calendar getLastUpdated() throws SQLException {
        Connection conn = null;
        PreparedStatement preparedStatement = null;
        ResultSet result = null;
        try {
            conn = DatabaseConnection.getConnection();
            preparedStatement = conn.prepareStatement("SELECT `last-updated` FROM `textblocks` WHERE page_ref = ? ORDER BY `last-updated` DESC LIMIT 1");
            preparedStatement.setString(1, uri);
            result = preparedStatement.executeQuery();
            Calendar calendar = Calendar.getInstance();
            if (result.next()) {
                java.sql.Timestamp timestamp = result.getTimestamp("last-updated");
                calendar.setTimeInMillis(timestamp.getTime());
            }
            return calendar;
        } finally {
            DatabaseConnection.closePreparedStatement(preparedStatement);
            DatabaseConnection.closeResultSet(result);
            DatabaseConnection.closeConnection(conn);
        }
    }
    public void editPage() throws SQLException {
        Connection conn = null;
        PreparedStatement preparedStatement = null;
        try {
            conn = DatabaseConnection.getConnection();

            // Edit the title.
            preparedStatement = conn.prepareStatement("UPDATE `pages` SET `title`=? WHERE uri=?");
            preparedStatement.setString(1, title);
            preparedStatement.setString(2, uri);
            preparedStatement.execute();

//            // Edit the number of columns.
//            preparedStatement = conn.prepareStatement("UPDATE `pages` SET `columns`=? WHERE uri=?");
//            preparedStatement.setInt(1, columns);
//            preparedStatement.setString(2, uri);
//            preparedStatement.execute();
//
//            // Edit the parent page.
//            preparedStatement = conn.prepareStatement("UPDATE `pages` SET `parent_page`=? WHERE uri=?");
//            preparedStatement.setString(1, this.parentPageUri);
//            preparedStatement.setString(2, uri);
//            preparedStatement.execute();
        } finally {
            DatabaseConnection.closePreparedStatement(preparedStatement);
            DatabaseConnection.closeConnection(conn);
        }
    }

    public void deletePage() throws SQLException {
        Connection conn = null;
        PreparedStatement preparedStatement = null;
        try {
            conn = DatabaseConnection.getConnection();
            preparedStatement = conn.prepareStatement("DELETE FROM `pages` WHERE uri=?");
            preparedStatement.setString(1, uri);
            preparedStatement.execute();
        } finally {
            DatabaseConnection.closePreparedStatement(preparedStatement);
            DatabaseConnection.closeConnection(conn);
        }
    }
    public static PageFolder getSortedPages() throws SQLException {
        Page[] pages = Page.getPages();
        PageFolder folder = new PageFolder("");
        String folderUri;
        PageFolder pagesFolder;
        for (Page page : pages) {
            folderUri = page.getFolderUri();
            pagesFolder = folder.getFolderByUri(folderUri);
            pagesFolder.add(page);
        }
        return folder;
    }
    /**
     * @return the identifier
     */
    public int getIdentifier() {
        return identifier;
    }

    /**
     * @return the name
     */
    public String getName() {
        return name;
    }

    /**
     * @param name the name to set
     */
    public void setName(String name) {
        this.name = name;
    }

    public String getFilename() {
        return this.getName();
    }

    public String getFolderUri() {
        if(REGEX_FILENAME==null)REGEX_FILENAME = Pattern.compile("^.*/([^/]*)$");
        Matcher matcher =REGEX_FILENAME.matcher(uri);
        String filename = "";
        if(matcher.find()) filename = matcher.group(1);
        String folder =  uri.replace(filename, "");
        return folder;
    }
}
